;==========
;Z80 Macros
;==========

;Loading registers
;-----------------

#define ld_HL_BC	ld h,b \ ld l,c
#define ld_BC_HL	ld h,b \ ld l,c
#define ld_HL_DE	ld h,d \ ld l,e
#define ld_DE_HL	ld h,d \ ld l,e
#define ld_HL_IX	push ix \ pop hl
#define ld_IX_HL	push hl \ pop ix
#define ld_HL_IY	push iy \ pop hl
#define ld_IY_HL	push hl \ pop iy

#define	PUSH_REGS		push af \ push bc \ push de \ push hl \ push ix \ push iy
#define	POP_REGS		pop iy \ pop ix \ pop hl \ pop de \ pop bc \ pop af
#define	EXX_REGS		ex af,af' \ exx

;Useful routines
;---------------
#define	ld_HL_ind		ld a,(hl) \ inc hl \ ld h,(hl) \ ld l,a
#define	ld_DE_HLind		ld e,(hl) \ inc hl \ ld d,(hl) \ inc hl
#define	cp_HL_BC		or a \ sbc hl,bc \ add hl,bc
#define	cp_HL_DE		or a \ sbc hl,de \ add hl,de
#define	cp_IX_HL		or a \ sbc ix,hl \ add ix,hl

;Replace undocumented instructions
;---------------------------------
;#define	sll(rr)   scf \ rl rr		;just wasted 4 clock cycles and 1 byte

;X86 syntax
#define mov ld

;Math implementation
;-------------------
#macro wsubHL(num)
	push bc
	ld bc,num
	or a
	sbc hl,bc
	pop bc
#endmacro
#define subHL wsubHL(

;Multiplication
;--------------
#define mul_add_reg8		add a,c
#define mul_transfer_reg8	ld a,c
#define mul_add_reg16		add hl,bc
#define mul_transfer_reg16	ld c,l \ ld b,h

#macro multA(constnum)

#endmacro
#macro multHL(constnum)

#endmacro 


;multHLconst(n)
#macro multHLfast( num )
#define mfstart 0
muloop(15)
#if mfstart=0 \ ld hl,0 \ #endif
#endmacro

#define muloop( loop ) #if loop >= 0\ do_mul( 1<<loop )\ muloop( eval( loop-1) )\ #endif

#macro do_mul(bit)
#if mfstart = 1
   add hl,hl
#endif
#if num&bit = bit
   #if mfstart = 0
     #if num&(bit-1)>0
       transfer_reg16
     #endif
     #define mfstart 1
   #else
     add_reg16
   #endif
  #endif
#endmacro

;Division
;--------
#macro divAconst(n)
;multiply A by n and divide by 256
 ld h,0
 ld l,a
 multHLconst(n)
 ld a,h
#endmacro

#macro divHLconst(n)
;multiply HL by n and divide by 256
 ld h,0
 ld l,a
 multHLconst(n)
 ld a,h
#endmacro   

;Tables


;VectorTable
#macro vector_table()


#endmacro

;JumpTable
#macro jump_table()


#endmacro

;API
#define SETPEN(xx,yy)		LDHL(yy,xx) \ ld (PenCol),hl
#define SETCUR(xx,yy)		LDHL(xx,yy) \ ld (currow),hl

;EASE Z80
#define COPY(src,dest,size)	ld hl,src \ ld de,dest \ ld bc,size \ ldir
#define FILLRAM(first,last,xx)	ld hl,first \ ld de,first+1 \ ld bc,last-first-1 \ ld (hl),xx \ ldir
#define ZERORAM(first,last)	FILLRAM(first,last,0)
#define RAMVAR(name,size)		name: \ .block size
#define MAX_A(xx)		cp xx \ jr nc,$+4 \ ld a,xx
#define MIN_A(xx)		cp xx \ jr c,$+4 \ ld a,xx

;C like functions

#macro malloc
;allocates memory

#endmacro
#macro memchr 
;searches an array for the first occurance of a character

#endmacro
#macro memcmp 
;compares two buffers

#endmacro
#macro memcpy 
;copies one buffer to another

#endmacro
#macro memmove 
;moves one buffer to another

#endmacro
#macro memset(buffer, size, num)
;fills a buffer with a character
	ld hl,buffer
	ld de,buffer+1
	ld (hl),num
	lb bc,size-1
	ldir
#endmacro

;random number
#macro srand()
;initialize the random number generator

#endmacro
#macro rand()
;returns a random number

#endmacro

;Strings
#macro substr()

#endmacro


;Added fake instructions
;-----------------------
.ADDINSTR ADC A,(HL+)	238E 2
.ADDINSTR ADC A,(HL-)	2B8E 2
.ADDINSTR ADD A,(HL+)	2386 2
.ADDINSTR ADD A,(HL-)	2B86 2
.ADDINSTR AND (HL+)	23A6 2
.ADDINSTR AND (HL-)	2BA6 2
.ADDINSTR DEC (HL+)	2335 2
.ADDINSTR DEC (HL-)	2B35 2
.ADDINSTR INC (HL+)	2334 2
.ADDINSTR INC (HL-)	2B34 2
.ADDINSTR LD (BC),A		02   1 NOP 1
.ADDINSTR LD (BC+),A	0302 2 NOP 1
.ADDINSTR LD (BC-),A	0B02 2 NOP 1
.ADDINSTR LD (DE),A		12   1 NOP 1
.ADDINSTR LD (DE+),A	1312 2 NOP 1
.ADDINSTR LD (DE-),A	1B12 2 NOP 1

.ADDINSTR LD (HL+),*	36  3 NOP 1 23
.ADDINSTR LD (HL-),*	36  3 NOP 1 2B
.ADDINSTR LD (HL),A		77   1 NOP 1
.ADDINSTR LD (HL+),A	2377 2 NOP 1
.ADDINSTR LD (HL-),A	2B77 2 NOP 1
.ADDINSTR LD (HL),B		70   1 NOP 1
.ADDINSTR LD (HL+),B	2370 2 NOP 1
.ADDINSTR LD (HL-),B	2B70 2 NOP 1
.ADDINSTR LD (HL),C		71   1 NOP 1
.ADDINSTR LD (HL+),C	2371 2 NOP 1
.ADDINSTR LD (HL-),C	2B71 2 NOP 1
.ADDINSTR LD (HL),D		72   1 NOP 1
.ADDINSTR LD (HL+),D	2372 2 NOP 1
.ADDINSTR LD (HL-),D	2B72 2 NOP 1
.ADDINSTR LD (HL),E		73   1 NOP 1
.ADDINSTR LD (HL+),E	2373 2 NOP 1
.ADDINSTR LD (HL-),E	2B73 2 NOP 1
.ADDINSTR LD (HL),H		74   1 NOP 1
.ADDINSTR LD (HL),L		75   1 NOP 1

.ADDINSTR LD   A,(BC+) 030A 2 NOP 1
.ADDINSTR LD   A,(BC-) 0B0A 2 NOP 1
.ADDINSTR LD   A,(HL+) 237E 2 NOP 1
.ADDINSTR LD   A,(HL-) 2B7E 2 NOP 1
.ADDINSTR LD   B,(HL+) 2346 2 NOP 1
.ADDINSTR LD   B,(HL-) 2B46 2 NOP 1
.ADDINSTR LD   C,(HL+) 234E 2 NOP 1
.ADDINSTR LD   C,(HL-) 2B4E 2 NOP 1
.ADDINSTR LD   D,(HL+) 2356 2 NOP 1
.ADDINSTR LD   D,(HL-) 2B56 2 NOP 1
.ADDINSTR LD   E,(HL+) 235E 2 NOP 1
.ADDINSTR LD   E,(HL-) 2B5E 2 NOP 1
.ADDINSTR LD   HL,A,(HL) 6F66237E 4 NOP 1
.ADDINSTR OR   (HL+)   23B6 2 NOP 1
.ADDINSTR OR   (HL-)   2BB6 2 NOP 1
.ADDINSTR RL   (HL+)   2316CB 3 NOP 1
.ADDINSTR RL   (HL-)   2B16CB 3 NOP 1
.ADDINSTR RR   (HL+)   231ECB 3 NOP 1
.ADDINSTR RR   (HL-)   2B1ECB 3 NOP 1
.ADDINSTR RRC  (HL+)   230ECB 3 NOP 1
.ADDINSTR RRC  (HL-)   2B0ECB 3 NOP 1
.ADDINSTR SBC  A,(HL+) 239E 2 NOP 1
.ADDINSTR SBC  A,(HL-) 2B9E 2 NOP 1
.ADDINSTR SLA  (HL+)   2326CB 3 NOP 1
.ADDINSTR SLA  (HL-)   2B26CB 3 NOP 1
.ADDINSTR SL1	(HL+)	2336CB	3	NOP	1
.ADDINSTR SL1	(HL-)	2B36CB	3	NOP	1
.ADDINSTR SLL	(HL+)	2336CB	3	NOP	1
.ADDINSTR SLL	(HL-)	2B36CB	3	NOP	1
.ADDINSTR SRA  (HL+)   232ECB 3 NOP 1
.ADDINSTR SRA  (HL-)   2B2ECB 3 NOP 1
.ADDINSTR SRL  (HL+)   233ECB 3 NOP 1
.ADDINSTR SRL  (HL-)   2B3ECB 3 NOP 1
.ADDINSTR SUB  (HL+)   2396 2 NOP 1
.ADDINSTR SUB  (HL-)   2B96 2 NOP 1
.ADDINSTR XOR  (HL+)   23AE 2 NOP 1
.ADDINSTR XOR  (HL-)   2BAE 2 NOP 1

.ADDINSTR LD HL,BC 6069 2 NOP 1
.ADDINSTR LD BC,HL 444D 2 NOP 1
.ADDINSTR LD HL,DE 626B 2 NOP 1
.ADDINSTR LD DE,HL 545d 2 NOP 1
.ADDINSTR LD DE,BC 5059 2 NOP 1
.ADDINSTR LD BC,DE 424B 2 NOP 1

;